[llvm-commits] CVS: llvm/lib/Bytecode/Writer/Writer.cpp WriterInternals.h

Chris Lattner lattner at cs.uiuc.edu
Fri Nov 11 17:33:52 PST 2005



Changes in directory llvm/lib/Bytecode/Writer:

Writer.cpp updated: 1.114 -> 1.115
WriterInternals.h updated: 1.25 -> 1.26
---
Log message:

Read and write section info from/to .bc files


---
Diffs of the changes:  (+38 -10)

 Writer.cpp        |   44 ++++++++++++++++++++++++++++++++++++++------
 WriterInternals.h |    4 ----
 2 files changed, 38 insertions(+), 10 deletions(-)


Index: llvm/lib/Bytecode/Writer/WriterInternals.h
diff -u llvm/lib/Bytecode/Writer/WriterInternals.h:1.25 llvm/lib/Bytecode/Writer/WriterInternals.h:1.26
--- llvm/lib/Bytecode/Writer/WriterInternals.h:1.25	Thu Apr 21 16:48:46 2005
+++ llvm/lib/Bytecode/Writer/WriterInternals.h	Fri Nov 11 19:33:40 2005
@@ -10,10 +10,6 @@
 // This header defines the interface used between components of the bytecode
 // writer.
 //
-// Note that the performance of this library is not terribly important, because
-// it shouldn't be used by JIT type applications... so it is not a huge focus
-// at least.  :)
-//
 //===----------------------------------------------------------------------===//
 
 #ifndef LLVM_LIB_BYTECODE_WRITER_WRITERINTERNALS_H






More information about the llvm-commits mailing list
Index: llvm/lib/Bytecode/Writer/Writer.cpp
diff -u llvm/lib/Bytecode/Writer/Writer.cpp:1.114 llvm/lib/Bytecode/Writer/Writer.cpp:1.115
--- llvm/lib/Bytecode/Writer/Writer.cpp:1.114	Fri Nov 11 19:01:50 2005
+++ llvm/lib/Bytecode/Writer/Writer.cpp	Fri Nov 11 19:33:40 2005
@@ -922,6 +922,11 @@
 void BytecodeWriter::outputModuleInfoBlock(const Module *M) {
   BytecodeBlock ModuleInfoBlock(BytecodeFormat::ModuleGlobalInfoBlockID, *this);
 
+  // Give numbers to sections as we encounter them.
+  unsigned SectionIDCounter = 0;
+  std::vector<std::string> SectionNames;
+  std::map<std::string, unsigned> SectionID;
+  
   // Output the types for the global variables in the module...
   for (Module::const_global_iterator I = M->global_begin(),
          End = M->global_end(); I != End; ++I) {
@@ -933,7 +938,7 @@
     
     // Fields: bit0 = isConstant, bit1 = hasInitializer, bit2-4=Linkage,
     // bit5+ = Slot # for type.
-    bool HasExtensionWord = I->getAlignment() != 0;
+    bool HasExtensionWord = (I->getAlignment() != 0) || I->hasSection();
     
     // If we need to use the extension byte, set linkage=3(internal) and
     // initializer = 0 (impossible!).
@@ -947,11 +952,22 @@
       output_vbr(oSlot);
       
       // The extension word has this format: bit 0 = has initializer, bit 1-3 =
       // linkage, bit 4-8 = alignment (log2), bits 10+ = future use.
       unsigned ExtWord = (unsigned)I->hasInitializer() |
                          (getEncodedLinkage(I) << 1) |
-                         ((Log2_32(I->getAlignment())+1) << 4);
+                         ((Log2_32(I->getAlignment())+1) << 4) |
+                         ((unsigned)I->hasSection() << 9);
       output_vbr(ExtWord);
+      if (I->hasSection()) {
+        // Give section names unique ID's.
+        unsigned &Entry = SectionID[I->getSection()];
+        if (Entry == 0) {
+          Entry = ++SectionIDCounter;
+          SectionNames.push_back(I->getSection());
+        }
+        output_vbr(Entry);
+      }
     }
 
     // If we have an initializer, output it now.
@@ -975,16 +991,27 @@
     if (I->isExternal())   // If external, we don't have an FunctionInfo block.
       ID |= 1 << 4;
     
-    if (I->getAlignment() || (CC & ~15) != 0)
+    if (I->getAlignment() || I->hasSection() || (CC & ~15) != 0)
       ID |= 1 << 31;       // Do we need an extension word?
     
     output_vbr(ID);
     
     if (ID & (1 << 31)) {
       // Extension byte: bits 0-4 = alignment, bits 5-9 = top nibble of calling
-      // convention.
-      ID = (Log2_32(I->getAlignment())+1) | ((CC >> 4) << 5);
+      // convention, bit 10 = hasSectionID.
+      ID = (Log2_32(I->getAlignment())+1) | ((CC >> 4) << 5) | 
+           (I->hasSection() << 10);
       output_vbr(ID);
+      
+      // Give section names unique ID's.
+      if (I->hasSection()) {
+        unsigned &Entry = SectionID[I->getSection()];
+        if (Entry == 0) {
+          Entry = ++SectionIDCounter;
+          SectionNames.push_back(I->getSection());
+        }
+        output_vbr(Entry);
+      }
     }
   }
   output_vbr((unsigned)Table.getSlot(Type::VoidTy) << 5);
@@ -998,6 +1025,11 @@
 
   // Output the target triple from the module
   output(M->getTargetTriple());
+  
+  // Emit the table of section names.
+  output_vbr((unsigned)SectionNames.size());
+  for (unsigned i = 0, e = SectionNames.size(); i != e; ++i)
+    output(SectionNames[i]);
 }
 
 void BytecodeWriter::outputInstructions(const Function *F) {
@@ -947,11 +952,22 @@
       
       // The extension word has this format: bit 0 = has initializer, bit 1-3 =
-      // linkage, bit 4-8 = alignment (log2), bits 10+ = future use.
+      // linkage, bit 4-8 = alignment (log2), bit 9 = has SectionID, 
+      // bits 10+ = future use.
       unsigned ExtWord = (unsigned)I->hasInitializer() |
                          (getEncodedLinkage(I) << 1) |

